#!/usr/bin/csh -f
# Copyright (C) 2000 Paul Ritchey <pritchey@arl.army.mil>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.




# Brief Description:
# This script is designed to be run through a cron job
# once an hour, every hour.

# It's purpose is to stop/restart the snort sensor and 
# tar/gzip hourly results.

# Version History:
# 08/03/2000        Version 1.0     First version of script.
#                                   Paul Ritchey, JASI

# BEGIN USER CONFIGURABLE SECTION
# BEGIN USER CONFIGURABLE SECTION
# BEGIN USER CONFIGURABLE SECTION

# This variable should be set to the name of
# the site being monitored.  It is used as part
# of the directory structure for the logs, and
# is also important to the 'pullback' script on
# the analysis box.
set sensor_site='MYSITE1'

# This variable should be set to the network interface
# you want the snort sensor process(es) to use.
set network_interface='hme0'

# This should be set to the number of days the data
# should reside on this server until it is deleted
# in order to conserve space.
set data_expires=1

# This variable should be set to the directory
# that contains the rule sets you want used for each 
# network block you want snort to watch.
set rules_directory='/home/snort/rules'

# This variable should be set to the directory where
# snort should log everything to.  It is the 'root' directory
# where snort will create hourly log directories, and then
# tar/gzip the hourly directories when the hour ends.
set log_directory='/home/snort/LOGS'

# This variable should be set to the directory and filename
# of the configuration file containing the network blocks
# snort should watch.
set network_list_file='/home/snort/network.cfg'

# END USER CONFIGURABLE SECTION
# END USER CONFIGURABLE SECTION
# END USER CONFIGURABLE SECTION

# Algorithm used:
#	1.  pkill the snort process(es)
#	2.  restart the snort process(es)
#		a.  Retrieve netblocks to watch.
#		b.  Create new directories
#			1.  Create new hour directory.
#			2.  Create new netblock subdirectories.:wq
#		c.  Start snort instance for each netblock.
#			1.  Determine CIDR info for current netblock
#			2.  Start instance of snort with appropriate params.
#	3.  Tar and gzip the last hour's data so it can be pulled
#	    down by the analyst box.
#	4.  Now delete data over a certain age (user specified)


set path=( /usr/bin /usr/sbin /usr/local/bin /bin )

# Due to some differences in the OS commands, we must
# determine the proper settings/commands for Linux.
# They vary slightly between Linux and Solaris.
if (`uname -s` == 'Linux') then
	set kill_command='killall'
	set tar_flags='cvfP'
else
	# Until others give me the proper settings
	# (if there are any other differences)
	# we assume OS is Solaris (SunOS).
	set kill_command='pkill'
	set tar_flags='cvf'
endif

# Now go read in the config file containing all of the 
# network blocks snort is to watch.
set network_to_watch=(`grep -v '#' $network_list_file`)

# Get the date information to that gets used for the creation of
# the hourly data.
set current_date=`date '+%Y%m%d'`
set current_hour=`date '+%H'`
set current_datehour=$current_date.$current_hour

#  Set up the previous hour date information.
if ($current_hour > '0') then
	set previous_date=$current_date
	@ previous_hour=$current_hour - 1
else
	@ previous_date=$current_date - 1
	set previous_hour=23
endif

# Make sure we have enough digits in the hour.
# Otherwise pad it with a '0'.
if ($previous_hour < '10') then
	set previous_hour = 0$previous_hour
endif
echo $previous_hour

set previous_datehour=$previous_date.$previous_hour

# Make the hourly logging directory.
mkdir $log_directory/$sensor_site/$current_datehour

#  First we need to kill all running snort processes.
$kill_command snort

# Since all scans are logged to the same log file, move it
# into the proper date directory.
mv $log_directory/$sensor_site/snort_portscan.log $log_directory/$sensor_site/$previous_datehour

# Now loop through each of the netblocks, creating the appropriate
# directories.
set myloop=1
while ( $myloop <= $#network_to_watch )
	# Make a netblock subdirectory in the hourly logging
	# directory.
	mkdir $log_directory/$sensor_site/$current_datehour/$network_to_watch[$myloop]

	# Determine CIDR bit count for current netblock.
	set myblock = $network_to_watch[$myloop]
	set myloop2 = 1
	set mybitcount = 0
	while ( $myloop2 <= 4)
		if ( $myblock:e == '0') then
			@ mybitcount +=8
		endif

		set myblock = $myblock:r
		@ myloop2++
	end

	# Start an instance of snort for the current netblock.
snort -A full -c $rules_directory/rules.$network_to_watch[$myloop] -d -D -e -h $network_to_watch[$myloop]/$mybitcount -i $network_interface -l $log_directory/$sensor_site/$current_datehour/$network_to_watch[$myloop]

	
	# Now increment our loop counter
	@ myloop++
end

# Now it's time to tar and gzip the previous hours data.
tar $tar_flags $log_directory/$sensor_site/$previous_datehour.tar $log_directory/$sensor_site/$previous_datehour
gzip $log_directory/$sensor_site/$previous_datehour.tar
rm -r $log_directory/$sensor_site/$previous_datehour

# Now delete data older than the user specified retension period.
find $log_directory/$sensor_site  -mtime $data_expires -exec rm -f {} \;

# We're done, so exit gracefully.
exit 0
