#!/usr/bin/csh -f
# Copyright (C) 2000 Paul Ritchey <pritchey@arl.army.mil>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.




# Brief Description:
# This script is designed to be run through a cron job
# once an hour, every hour.

# It's purpose is to retrieve the hourly wrapup from
# the snort sensor to the analysis box.  Called
# without any parameter it will retrieve the 
# previous hour's wrapup from the sensor.  Called with
# a parameter it will retrieve the requested hour's
# wrapup data from the sensor.

# After retrieving the hourly wrapup, it will process
# the data (after untarring and ungzipping it) through
# SnortSnarf to create the html pages for the analyst(s).

# Version History:
# 08/04/2000        Version 1.0     First version of script.
#                                   Paul Ritchey, JASI

# BEGIN USER CONFIGURABLE SECTION
# BEGIN USER CONFIGURABLE SECTION
# BEGIN USER CONFIGURABLE SECTION

# This should be set to the directory where
# all sensor data retrieved from the sensor
# is to reside.  This script will automatically
# place the data into the appropriate site 
# subdirectory in this directory.  It is also
# used when retrieving data from the sensor.
set log_directory='/home/snort/LOGS'

# This should be set to the number of days the data
# should reside on this server until it is deleted
# in order to conserve space.
set data_expires=2

# This variable should be set to the path where the
# SnortSnarf perl script can be found ('snortsnarf.pl').
# Or you can use this to point to whatever script you 
# choose to use for processing the retrieved log data.
set snortsnarf_path='/home/snort/scripts'

# This variable should be set to the path where
# the GNU 'date' command resides.
# NOTE:  GNU 'date' COMES WITH LINUX, BUT NOT OTHER
# FLAVORS OF UNIX. WHEN DEALING WITH MULTIPLE PLATFORMS
# YOU MAY WANT TO 'ln -s /usr/bin/date /usr/local/date'
# ON THE LINUX BOXES.
set gnudate_path='/usr/local/bin'

# This is the account to use when scp'ing the wrapup
# files from the sensors.
set retrieve_account='root'

# END USER CONFIGURABLE SECTION
# END USER CONFIGURABLE SECTION
# END USER CONFIGURABLE SECTION

# Algorithm used:
#	1.  Make sure we have enough parameters to run with.
#		a. site name (used for directory name on remote box)
#		b. sensor name or ip.
#		c. + | - for time offset
#		d. number to be added/subtracted from local time
#			NOTE:  ITEMS C AND D WILL BE
#			IGNORED WHEN USED MANUALLY TO RETRIEVE DATA FROM COMMAND LINE.
#		e. OPTIONAL: date.hour to retrieve by hand.
#			Format: yyyymmdd.hh
#			Example: 20000801.13 retrieves 08/01/2000 at 1300
#	2.  If we passed step 1 ok, go retrieve data.
#		a.  If using 'manual' mode, retrieve the requested hour's
#		    data the user is manually retrieving.
#		b.  If using 'auto' mode, get date/time and retrieve
#		    previous hour's data.
#	3.  Gunzip/untar retrieved data, then delete file.
#	4.  Call script to process data.
#	5.  Delete directories/files over certain age (user specified).


set path=( /usr/bin /usr/sbin /usr/local/bin /bin )

# Check for number of parameters, we need at least 2 to get started.
if ($#argv < 2) then
	echo ""
	echo "Incorrect number of parameters passed.  Usage:"
	echo ""
	echo "retrieve_wrapup.sh <site_name> <hostname | ip> [yyyymmdd.hh]"
	echo ""
	echo ""
	exit 0
endif

# Determine the date/time we need to retrieve if the user
# hasn't passed it in to us.
if ($#argv > 2) then
	set previous_datehour=$argv[3]
	set previous_date=$argv[3]:r
else
	set previous_datehour=`ssh $retrieve_account@{$argv[2]} "$gnudate_path/date --date='1 hour ago' '+%Y%m%d.%H'"`
	set previous_date=$previous_datehour:r
endif

# Test to see if the site logging directory exists
# here on the analyst box.
if !( -e $log_directory/$argv[1] ) then
	mkdir $log_directory/$argv[1]

	# Make sure it was created, if not, notify user
	# and exit.
	if !( -e $log_directory/$argv[1] ) then
		echo " "
		echo "Unable to continue....."
		echo "Unable to create needed site parent directory:"
		echo "    "$log_directory/$argv[1]
		echo " "
		exit 0
	endif
endif

# Now test for the date subdirectory we are retrieving
# data for.
if !( -e $log_directory/$argv[1]/$previous_date ) then
        mkdir $log_directory/$argv[1]/$previous_date

        # Make sure it was created, if not, notify user
        # and exit.
        if !( -e $log_directory/$argv[1]/$previous_date ) then
                echo " "
                echo "Unable to continue....."
                echo "Unable to create needed site date directory:"
                echo "    "$log_directory/$argv[1]/$previous_date
                echo " "
                exit 0
        endif
endif

# Go to the proper directory before we retrieve the sensor data.
cd $log_directory/$argv[1]/$previous_date

# Now retrieve the data from the sensor via scp.
scp $retrieve_account@{$argv[2]}:$log_directory/$argv[1]/$previous_datehour.tar.gz $previous_datehour.tar.gz

# Gunzip and untar the file.....
gunzip  -c $previous_datehour.tar.gz | tar xvf -

# Now move the resulting directory into the proper spot.
mv $log_directory/$argv[1]/$previous_datehour $log_directory/$argv[1]/$previous_date

# Delete the scp'ed file.
rm $previous_datehour.tar.gz

# Now cd into the directory we are processing....
cd $log_directory/$argv[1]/$previous_date/$previous_datehour

# Get a directory listing.  We need to make sure we process all
# subnets that may be at this site.
set subnet_list=`ls`

# Create the directory that will hold ALL of the subnet
# log data.
mkdir snort_logs

# Walk through each item in the directory listing.
set myloop=1
while ($myloop <= $#subnet_list)
	# See if the item is a directory.
	if (-d $subnet_list[$myloop]) then
		cat $subnet_list[$myloop]/snort.alert >> snort.alert
		cat $subnet_list[$myloop]/TELNET >> TELNET
		cp -rf $subnet_list[$myloop]/* snort_logs
		rm -rf $subnet_list[$myloop]
	endif

	# increment the counter.
	@ myloop++
end

# Delete the bogus snort.alert file that results from doing
# the copies in the while loop
rm -f snort_logs/snort.alert
rm -f snort_logs/TELNET

# Open up the permissions show people can see the log
# entries down at the packet level.
chmod -R 755 snort_logs

# The final and last step, process the date and create
# the web pages!!!
$snortsnarf_path/snortsnarf.pl -d $log_directory/$argv[1]/$previous_date/$previous_datehour.html -ldir /snort/$argv[1]/$previous_date/$previous_datehour/snort_logs $log_directory/$argv[1]/$previous_date/$previous_datehour/snort.alert $log_directory/$argv[1]/$previous_date/$previous_datehour/snort_portscan.log

# Ok.  I was kidding.  One last step to go......
# We need to create an index page of all of the hours
# that are available for browsing.....
cd $log_directory/$argv[1]/$previous_date
#set available_hours=`ls | grep -v html`

# Echo out an 'index' page that contains the nav bar for the current date.
rm -f index.html
echo '<HTML>' >> index.html
echo '<HEAD>' >> index.html
echo '<TITLE>'$argv[1]' Snort Results</TITLE>' >> index.html
echo '</HEAD>' >> index.html
echo '<FRAMESET ROWS="95,*" BORDER=10 FRAMEBORDER=7>' >> index.html
echo '<FRAME SRC="navbar.html" NAME="navbar" SCROLLING=NO NORESIZE>' >> index.html
echo '<FRAME SRC="'$previous_date'.00.html/index.html" NAME="data" SCROLLING=YES>' >> index.html
echo '</FRAMESET>' >> index.html
echo '</HTML>' >> index.html

# Echo out the navbar for the current date.
rm -f navbar.html
echo '<HTML>' >> navbar.html
echo '<BODY BGCOLOR="#bebebe">' >> navbar.html
echo '<TABLE ALIGN=CENTER BORDER=5 BORDERCOLOR="#0000" CELLSPACING="5" CELLPADDING="5">' >> navbar.html
echo '<TR>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A><B>AM</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.00.html" TARGET="data"><B>00</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.01.html" TARGET="data"><B>01</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.02.html" TARGET="data"><B>02</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.03.html" TARGET="data"><B>03</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.04.html" TARGET="data"><B>04</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.05.html" TARGET="data"><B>05</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.06.html" TARGET="data"><B>06</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.07.html" TARGET="data"><B>07</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.08.html" TARGET="data"><B>08</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.09.html" TARGET="data"><B>09</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.10.html" TARGET="data"><B>10</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.11.html" TARGET="data"><B>11</B></A></TD></TR>' >> navbar.html
echo '<TR>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A><B>PM</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.12.html" TARGET="data"><B>12</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.13.html" TARGET="data"><B>13</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.14.html" TARGET="data"><B>14</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.15.html" TARGET="data"><B>15</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.16.html" TARGET="data"><B>16</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.17.html" TARGET="data"><B>17</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.18.html" TARGET="data"><B>18</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.19.html" TARGET="data"><B>19</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.20.html" TARGET="data"><B>20</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.21.html" TARGET="data"><B>21</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.22.html" TARGET="data"><B>22</B></A></TD>' >> navbar.html
echo '<TD ALIGN=CENTER VALIGN=MIDDLE><A HREF="'$previous_date'.23.html" TARGET="data"><B>23</B></A></TD>' >> navbar.html
echo '</TR>' >> navbar.html
echo '</TABLE>' >> navbar.html
echo '</BODY>' >> navbar.html
echo '</HTML>' >> navbar.html


# Now delete data older than the user specified retension period.
find $log_directory/$argv[1]  -mtime $data_expires -exec rm -rf {} \;

# We're done, so exit gracefully.
exit 0
